# Generated from D:/Projects/BESSER/besser/BUML/notations/action_language/BESSERActionLanguage.g4 by ANTLR 4.13.2
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,57,594,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,2,6,7,
        6,2,7,7,7,2,8,7,8,2,9,7,9,2,10,7,10,2,11,7,11,2,12,7,12,2,13,7,13,
        2,14,7,14,2,15,7,15,2,16,7,16,2,17,7,17,2,18,7,18,2,19,7,19,2,20,
        7,20,2,21,7,21,2,22,7,22,2,23,7,23,2,24,7,24,2,25,7,25,2,26,7,26,
        2,27,7,27,2,28,7,28,2,29,7,29,2,30,7,30,2,31,7,31,2,32,7,32,2,33,
        7,33,2,34,7,34,2,35,7,35,2,36,7,36,2,37,7,37,2,38,7,38,2,39,7,39,
        2,40,7,40,2,41,7,41,2,42,7,42,2,43,7,43,2,44,7,44,2,45,7,45,2,46,
        7,46,2,47,7,47,2,48,7,48,2,49,7,49,2,50,7,50,2,51,7,51,2,52,7,52,
        2,53,7,53,2,54,7,54,2,55,7,55,2,56,7,56,2,57,7,57,2,58,7,58,2,59,
        7,59,1,0,1,0,1,0,1,0,1,0,1,0,5,0,127,8,0,10,0,12,0,130,9,0,1,0,1,
        0,5,0,134,8,0,10,0,12,0,137,9,0,1,0,1,0,1,0,5,0,142,8,0,10,0,12,
        0,145,9,0,3,0,147,8,0,1,0,1,0,1,0,3,0,152,8,0,1,0,1,0,1,1,1,1,1,
        1,1,1,1,2,1,2,1,2,1,2,1,2,1,2,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,3,1,
        3,3,3,175,8,3,1,4,1,4,3,4,179,8,4,1,5,1,5,1,5,1,5,1,5,1,5,1,6,1,
        6,1,6,1,6,1,6,1,6,1,6,1,7,1,7,1,7,1,7,1,7,5,7,199,8,7,10,7,12,7,
        202,9,7,1,7,1,7,1,7,1,8,1,8,1,8,1,8,1,9,1,9,3,9,213,8,9,1,10,1,10,
        5,10,217,8,10,10,10,12,10,220,9,10,1,10,1,10,1,11,1,11,1,11,1,11,
        1,11,1,11,1,11,3,11,231,8,11,1,12,1,12,1,12,1,12,3,12,237,8,12,1,
        13,1,13,1,13,1,13,1,14,1,14,1,14,1,14,1,14,1,15,1,15,1,15,1,15,1,
        16,1,16,1,16,3,16,255,8,16,1,17,1,17,1,17,1,17,1,17,1,17,1,17,3,
        17,264,8,17,1,18,1,18,1,18,1,19,1,19,1,19,1,20,1,20,1,20,1,20,5,
        20,276,8,20,10,20,12,20,279,9,20,1,20,1,20,5,20,283,8,20,10,20,12,
        20,286,9,20,1,20,1,20,1,20,5,20,291,8,20,10,20,12,20,294,9,20,3,
        20,296,8,20,1,20,1,20,1,20,1,20,1,21,1,21,1,22,1,22,1,23,1,23,1,
        24,1,24,1,25,1,25,1,26,1,26,1,27,1,27,1,28,1,28,1,29,1,29,1,29,1,
        29,1,29,1,29,1,29,1,29,1,29,5,29,327,8,29,10,29,12,29,330,9,29,1,
        30,1,30,1,31,1,31,1,31,1,31,1,31,1,31,5,31,340,8,31,10,31,12,31,
        343,9,31,1,32,1,32,1,32,1,32,1,32,1,32,5,32,351,8,32,10,32,12,32,
        354,9,32,1,33,1,33,1,33,1,33,1,33,1,33,5,33,362,8,33,10,33,12,33,
        365,9,33,1,34,1,34,1,34,1,34,1,34,1,34,5,34,373,8,34,10,34,12,34,
        376,9,34,1,35,1,35,1,35,1,35,1,35,1,35,5,35,384,8,35,10,35,12,35,
        387,9,35,1,36,1,36,1,37,1,37,1,37,1,37,1,37,1,37,5,37,397,8,37,10,
        37,12,37,400,9,37,1,38,1,38,1,38,1,38,1,38,1,38,5,38,408,8,38,10,
        38,12,38,411,9,38,1,39,1,39,1,39,1,39,1,39,1,39,5,39,419,8,39,10,
        39,12,39,422,9,39,1,40,1,40,1,40,1,40,1,40,1,40,1,40,1,40,1,40,1,
        40,1,40,3,40,435,8,40,1,40,1,40,1,40,5,40,440,8,40,10,40,12,40,443,
        9,40,1,41,1,41,1,41,1,41,1,41,1,41,1,41,1,41,1,41,1,41,1,41,1,41,
        1,41,1,41,1,41,1,41,1,41,1,41,5,41,463,8,41,10,41,12,41,466,9,41,
        3,41,468,8,41,1,41,5,41,471,8,41,10,41,12,41,474,9,41,1,42,1,42,
        1,42,1,42,1,43,1,43,1,43,1,43,1,43,1,44,1,44,1,44,1,44,1,44,1,44,
        1,44,5,44,492,8,44,10,44,12,44,495,9,44,3,44,497,8,44,1,44,1,44,
        1,45,1,45,1,45,1,45,1,45,1,45,1,45,1,45,1,45,3,45,510,8,45,1,46,
        1,46,1,46,1,46,1,46,5,46,517,8,46,10,46,12,46,520,9,46,3,46,522,
        8,46,1,46,1,46,1,47,1,47,1,48,1,48,1,48,1,48,1,48,1,48,5,48,534,
        8,48,10,48,12,48,537,9,48,3,48,539,8,48,1,48,1,48,1,49,1,49,1,49,
        3,49,546,8,49,1,50,1,50,1,50,1,50,1,50,1,50,3,50,554,8,50,1,51,1,
        51,1,52,1,52,1,53,1,53,1,54,1,54,1,55,1,55,1,56,1,56,1,56,1,56,1,
        57,1,57,1,57,1,57,1,57,5,57,575,8,57,10,57,12,57,578,9,57,3,57,580,
        8,57,1,57,1,57,1,58,1,58,1,58,1,58,1,58,1,58,1,58,1,58,1,59,1,59,
        1,59,0,11,58,62,64,66,68,70,74,76,78,80,82,60,0,2,4,6,8,10,12,14,
        16,18,20,22,24,26,28,30,32,34,36,38,40,42,44,46,48,50,52,54,56,58,
        60,62,64,66,68,70,72,74,76,78,80,82,84,86,88,90,92,94,96,98,100,
        102,104,106,108,110,112,114,116,118,0,5,1,0,30,31,1,0,32,35,1,0,
        37,38,1,0,39,40,1,0,47,48,604,0,120,1,0,0,0,2,155,1,0,0,0,4,159,
        1,0,0,0,6,174,1,0,0,0,8,178,1,0,0,0,10,180,1,0,0,0,12,186,1,0,0,
        0,14,193,1,0,0,0,16,206,1,0,0,0,18,212,1,0,0,0,20,214,1,0,0,0,22,
        223,1,0,0,0,24,236,1,0,0,0,26,238,1,0,0,0,28,242,1,0,0,0,30,247,
        1,0,0,0,32,254,1,0,0,0,34,263,1,0,0,0,36,265,1,0,0,0,38,268,1,0,
        0,0,40,271,1,0,0,0,42,301,1,0,0,0,44,303,1,0,0,0,46,305,1,0,0,0,
        48,307,1,0,0,0,50,309,1,0,0,0,52,311,1,0,0,0,54,313,1,0,0,0,56,315,
        1,0,0,0,58,317,1,0,0,0,60,331,1,0,0,0,62,333,1,0,0,0,64,344,1,0,
        0,0,66,355,1,0,0,0,68,366,1,0,0,0,70,377,1,0,0,0,72,388,1,0,0,0,
        74,390,1,0,0,0,76,401,1,0,0,0,78,412,1,0,0,0,80,434,1,0,0,0,82,444,
        1,0,0,0,84,475,1,0,0,0,86,479,1,0,0,0,88,484,1,0,0,0,90,509,1,0,
        0,0,92,511,1,0,0,0,94,525,1,0,0,0,96,527,1,0,0,0,98,545,1,0,0,0,
        100,553,1,0,0,0,102,555,1,0,0,0,104,557,1,0,0,0,106,559,1,0,0,0,
        108,561,1,0,0,0,110,563,1,0,0,0,112,565,1,0,0,0,114,569,1,0,0,0,
        116,583,1,0,0,0,118,591,1,0,0,0,120,121,5,1,0,0,121,122,5,52,0,0,
        122,146,5,2,0,0,123,128,3,2,1,0,124,125,5,3,0,0,125,127,3,2,1,0,
        126,124,1,0,0,0,127,130,1,0,0,0,128,126,1,0,0,0,128,129,1,0,0,0,
        129,135,1,0,0,0,130,128,1,0,0,0,131,132,5,3,0,0,132,134,3,4,2,0,
        133,131,1,0,0,0,134,137,1,0,0,0,135,133,1,0,0,0,135,136,1,0,0,0,
        136,147,1,0,0,0,137,135,1,0,0,0,138,143,3,4,2,0,139,140,5,3,0,0,
        140,142,3,4,2,0,141,139,1,0,0,0,142,145,1,0,0,0,143,141,1,0,0,0,
        143,144,1,0,0,0,144,147,1,0,0,0,145,143,1,0,0,0,146,123,1,0,0,0,
        146,138,1,0,0,0,146,147,1,0,0,0,147,148,1,0,0,0,148,151,5,4,0,0,
        149,150,5,5,0,0,150,152,3,32,16,0,151,149,1,0,0,0,151,152,1,0,0,
        0,152,153,1,0,0,0,153,154,3,20,10,0,154,1,1,0,0,0,155,156,5,52,0,
        0,156,157,5,6,0,0,157,158,3,32,16,0,158,3,1,0,0,0,159,160,5,52,0,
        0,160,161,5,6,0,0,161,162,3,32,16,0,162,163,5,7,0,0,163,164,3,56,
        28,0,164,5,1,0,0,0,165,175,3,8,4,0,166,175,3,14,7,0,167,175,3,22,
        11,0,168,175,3,30,15,0,169,175,3,0,0,0,170,175,3,28,14,0,171,172,
        3,56,28,0,172,173,5,8,0,0,173,175,1,0,0,0,174,165,1,0,0,0,174,166,
        1,0,0,0,174,167,1,0,0,0,174,168,1,0,0,0,174,169,1,0,0,0,174,170,
        1,0,0,0,174,171,1,0,0,0,175,7,1,0,0,0,176,179,3,10,5,0,177,179,3,
        12,6,0,178,176,1,0,0,0,178,177,1,0,0,0,179,9,1,0,0,0,180,181,5,9,
        0,0,181,182,5,2,0,0,182,183,3,56,28,0,183,184,5,4,0,0,184,185,3,
        20,10,0,185,11,1,0,0,0,186,187,5,10,0,0,187,188,3,20,10,0,188,189,
        5,9,0,0,189,190,5,2,0,0,190,191,3,56,28,0,191,192,5,4,0,0,192,13,
        1,0,0,0,193,194,5,11,0,0,194,195,5,2,0,0,195,200,3,16,8,0,196,197,
        5,3,0,0,197,199,3,16,8,0,198,196,1,0,0,0,199,202,1,0,0,0,200,198,
        1,0,0,0,200,201,1,0,0,0,201,203,1,0,0,0,202,200,1,0,0,0,203,204,
        5,4,0,0,204,205,3,20,10,0,205,15,1,0,0,0,206,207,3,118,59,0,207,
        208,5,12,0,0,208,209,3,56,28,0,209,17,1,0,0,0,210,213,3,20,10,0,
        211,213,3,22,11,0,212,210,1,0,0,0,212,211,1,0,0,0,213,19,1,0,0,0,
        214,218,5,13,0,0,215,217,3,6,3,0,216,215,1,0,0,0,217,220,1,0,0,0,
        218,216,1,0,0,0,218,219,1,0,0,0,219,221,1,0,0,0,220,218,1,0,0,0,
        221,222,5,14,0,0,222,21,1,0,0,0,223,224,5,15,0,0,224,225,5,2,0,0,
        225,226,3,56,28,0,226,227,5,4,0,0,227,230,3,20,10,0,228,229,5,16,
        0,0,229,231,3,18,9,0,230,228,1,0,0,0,230,231,1,0,0,0,231,23,1,0,
        0,0,232,237,3,84,42,0,233,237,3,86,43,0,234,237,3,118,59,0,235,237,
        3,26,13,0,236,232,1,0,0,0,236,233,1,0,0,0,236,234,1,0,0,0,236,235,
        1,0,0,0,237,25,1,0,0,0,238,239,5,52,0,0,239,240,5,6,0,0,240,241,
        3,32,16,0,241,27,1,0,0,0,242,243,3,24,12,0,243,244,5,7,0,0,244,245,
        3,56,28,0,245,246,5,8,0,0,246,29,1,0,0,0,247,248,5,17,0,0,248,249,
        3,56,28,0,249,250,5,8,0,0,250,31,1,0,0,0,251,255,3,34,17,0,252,255,
        3,36,18,0,253,255,3,40,20,0,254,251,1,0,0,0,254,252,1,0,0,0,254,
        253,1,0,0,0,255,33,1,0,0,0,256,264,3,42,21,0,257,264,3,44,22,0,258,
        264,3,46,23,0,259,264,3,48,24,0,260,264,3,50,25,0,261,264,3,52,26,
        0,262,264,3,54,27,0,263,256,1,0,0,0,263,257,1,0,0,0,263,258,1,0,
        0,0,263,259,1,0,0,0,263,260,1,0,0,0,263,261,1,0,0,0,263,262,1,0,
        0,0,264,35,1,0,0,0,265,266,3,34,17,0,266,267,5,18,0,0,267,37,1,0,
        0,0,268,269,3,32,16,0,269,270,5,19,0,0,270,39,1,0,0,0,271,295,5,
        20,0,0,272,277,3,32,16,0,273,274,5,3,0,0,274,276,3,32,16,0,275,273,
        1,0,0,0,276,279,1,0,0,0,277,275,1,0,0,0,277,278,1,0,0,0,278,284,
        1,0,0,0,279,277,1,0,0,0,280,281,5,3,0,0,281,283,3,38,19,0,282,280,
        1,0,0,0,283,286,1,0,0,0,284,282,1,0,0,0,284,285,1,0,0,0,285,296,
        1,0,0,0,286,284,1,0,0,0,287,292,3,38,19,0,288,289,5,3,0,0,289,291,
        3,38,19,0,290,288,1,0,0,0,291,294,1,0,0,0,292,290,1,0,0,0,292,293,
        1,0,0,0,293,296,1,0,0,0,294,292,1,0,0,0,295,272,1,0,0,0,295,287,
        1,0,0,0,295,296,1,0,0,0,296,297,1,0,0,0,297,298,5,21,0,0,298,299,
        5,5,0,0,299,300,3,32,16,0,300,41,1,0,0,0,301,302,5,22,0,0,302,43,
        1,0,0,0,303,304,5,23,0,0,304,45,1,0,0,0,305,306,5,24,0,0,306,47,
        1,0,0,0,307,308,5,25,0,0,308,49,1,0,0,0,309,310,5,26,0,0,310,51,
        1,0,0,0,311,312,5,27,0,0,312,53,1,0,0,0,313,314,5,52,0,0,314,55,
        1,0,0,0,315,316,3,58,29,0,316,57,1,0,0,0,317,318,6,29,-1,0,318,319,
        3,60,30,0,319,328,1,0,0,0,320,321,10,2,0,0,321,322,5,19,0,0,322,
        323,3,56,28,0,323,324,5,6,0,0,324,325,3,56,28,0,325,327,1,0,0,0,
        326,320,1,0,0,0,327,330,1,0,0,0,328,326,1,0,0,0,328,329,1,0,0,0,
        329,59,1,0,0,0,330,328,1,0,0,0,331,332,3,62,31,0,332,61,1,0,0,0,
        333,334,6,31,-1,0,334,335,3,64,32,0,335,341,1,0,0,0,336,337,10,2,
        0,0,337,338,5,28,0,0,338,340,3,64,32,0,339,336,1,0,0,0,340,343,1,
        0,0,0,341,339,1,0,0,0,341,342,1,0,0,0,342,63,1,0,0,0,343,341,1,0,
        0,0,344,345,6,32,-1,0,345,346,3,66,33,0,346,352,1,0,0,0,347,348,
        10,2,0,0,348,349,5,29,0,0,349,351,3,66,33,0,350,347,1,0,0,0,351,
        354,1,0,0,0,352,350,1,0,0,0,352,353,1,0,0,0,353,65,1,0,0,0,354,352,
        1,0,0,0,355,356,6,33,-1,0,356,357,3,68,34,0,357,363,1,0,0,0,358,
        359,10,2,0,0,359,360,7,0,0,0,360,362,3,68,34,0,361,358,1,0,0,0,362,
        365,1,0,0,0,363,361,1,0,0,0,363,364,1,0,0,0,364,67,1,0,0,0,365,363,
        1,0,0,0,366,367,6,34,-1,0,367,368,3,70,35,0,368,374,1,0,0,0,369,
        370,10,2,0,0,370,371,7,1,0,0,371,373,3,70,35,0,372,369,1,0,0,0,373,
        376,1,0,0,0,374,372,1,0,0,0,374,375,1,0,0,0,375,69,1,0,0,0,376,374,
        1,0,0,0,377,378,6,35,-1,0,378,379,3,72,36,0,379,385,1,0,0,0,380,
        381,10,2,0,0,381,382,5,36,0,0,382,384,3,54,27,0,383,380,1,0,0,0,
        384,387,1,0,0,0,385,383,1,0,0,0,385,386,1,0,0,0,386,71,1,0,0,0,387,
        385,1,0,0,0,388,389,3,74,37,0,389,73,1,0,0,0,390,391,6,37,-1,0,391,
        392,3,76,38,0,392,398,1,0,0,0,393,394,10,2,0,0,394,395,7,2,0,0,395,
        397,3,76,38,0,396,393,1,0,0,0,397,400,1,0,0,0,398,396,1,0,0,0,398,
        399,1,0,0,0,399,75,1,0,0,0,400,398,1,0,0,0,401,402,6,38,-1,0,402,
        403,3,78,39,0,403,409,1,0,0,0,404,405,10,2,0,0,405,406,7,3,0,0,406,
        408,3,78,39,0,407,404,1,0,0,0,408,411,1,0,0,0,409,407,1,0,0,0,409,
        410,1,0,0,0,410,77,1,0,0,0,411,409,1,0,0,0,412,413,6,39,-1,0,413,
        414,3,80,40,0,414,420,1,0,0,0,415,416,10,2,0,0,416,417,5,41,0,0,
        417,419,3,80,40,0,418,415,1,0,0,0,419,422,1,0,0,0,420,418,1,0,0,
        0,420,421,1,0,0,0,421,79,1,0,0,0,422,420,1,0,0,0,423,424,6,40,-1,
        0,424,435,3,82,41,0,425,426,5,38,0,0,426,435,3,56,28,0,427,428,5,
        42,0,0,428,435,3,56,28,0,429,430,5,2,0,0,430,431,3,54,27,0,431,432,
        5,4,0,0,432,433,3,56,28,0,433,435,1,0,0,0,434,423,1,0,0,0,434,425,
        1,0,0,0,434,427,1,0,0,0,434,429,1,0,0,0,435,441,1,0,0,0,436,437,
        10,1,0,0,437,438,5,43,0,0,438,440,3,56,28,0,439,436,1,0,0,0,440,
        443,1,0,0,0,441,439,1,0,0,0,441,442,1,0,0,0,442,81,1,0,0,0,443,441,
        1,0,0,0,444,445,6,41,-1,0,445,446,3,90,45,0,446,472,1,0,0,0,447,
        448,10,3,0,0,448,449,5,44,0,0,449,471,5,52,0,0,450,451,10,2,0,0,
        451,452,5,20,0,0,452,453,3,56,28,0,453,454,5,21,0,0,454,471,1,0,
        0,0,455,456,10,1,0,0,456,457,5,44,0,0,457,458,5,52,0,0,458,467,5,
        2,0,0,459,464,3,56,28,0,460,461,5,3,0,0,461,463,3,56,28,0,462,460,
        1,0,0,0,463,466,1,0,0,0,464,462,1,0,0,0,464,465,1,0,0,0,465,468,
        1,0,0,0,466,464,1,0,0,0,467,459,1,0,0,0,467,468,1,0,0,0,468,469,
        1,0,0,0,469,471,5,4,0,0,470,447,1,0,0,0,470,450,1,0,0,0,470,455,
        1,0,0,0,471,474,1,0,0,0,472,470,1,0,0,0,472,473,1,0,0,0,473,83,1,
        0,0,0,474,472,1,0,0,0,475,476,3,82,41,0,476,477,5,44,0,0,477,478,
        5,52,0,0,478,85,1,0,0,0,479,480,3,82,41,0,480,481,5,20,0,0,481,482,
        3,56,28,0,482,483,5,21,0,0,483,87,1,0,0,0,484,485,3,82,41,0,485,
        486,5,44,0,0,486,487,5,52,0,0,487,496,5,2,0,0,488,493,3,56,28,0,
        489,490,5,3,0,0,490,492,3,56,28,0,491,489,1,0,0,0,492,495,1,0,0,
        0,493,491,1,0,0,0,493,494,1,0,0,0,494,497,1,0,0,0,495,493,1,0,0,
        0,496,488,1,0,0,0,496,497,1,0,0,0,497,498,1,0,0,0,498,499,5,4,0,
        0,499,89,1,0,0,0,500,501,5,2,0,0,501,502,3,56,28,0,502,503,5,4,0,
        0,503,510,1,0,0,0,504,510,3,92,46,0,505,510,3,98,49,0,506,510,3,
        94,47,0,507,510,3,96,48,0,508,510,3,118,59,0,509,500,1,0,0,0,509,
        504,1,0,0,0,509,505,1,0,0,0,509,506,1,0,0,0,509,507,1,0,0,0,509,
        508,1,0,0,0,510,91,1,0,0,0,511,512,5,52,0,0,512,521,5,2,0,0,513,
        518,3,56,28,0,514,515,5,3,0,0,515,517,3,56,28,0,516,514,1,0,0,0,
        517,520,1,0,0,0,518,516,1,0,0,0,518,519,1,0,0,0,519,522,1,0,0,0,
        520,518,1,0,0,0,521,513,1,0,0,0,521,522,1,0,0,0,522,523,1,0,0,0,
        523,524,5,4,0,0,524,93,1,0,0,0,525,526,5,45,0,0,526,95,1,0,0,0,527,
        528,5,46,0,0,528,529,3,54,27,0,529,538,5,2,0,0,530,535,3,56,28,0,
        531,532,5,3,0,0,532,534,3,56,28,0,533,531,1,0,0,0,534,537,1,0,0,
        0,535,533,1,0,0,0,535,536,1,0,0,0,536,539,1,0,0,0,537,535,1,0,0,
        0,538,530,1,0,0,0,538,539,1,0,0,0,539,540,1,0,0,0,540,541,5,4,0,
        0,541,97,1,0,0,0,542,546,3,100,50,0,543,546,3,114,57,0,544,546,3,
        116,58,0,545,542,1,0,0,0,545,543,1,0,0,0,545,544,1,0,0,0,546,99,
        1,0,0,0,547,554,3,102,51,0,548,554,3,104,52,0,549,554,3,106,53,0,
        550,554,3,108,54,0,551,554,3,110,55,0,552,554,3,112,56,0,553,547,
        1,0,0,0,553,548,1,0,0,0,553,549,1,0,0,0,553,550,1,0,0,0,553,551,
        1,0,0,0,553,552,1,0,0,0,554,101,1,0,0,0,555,556,5,53,0,0,556,103,
        1,0,0,0,557,558,5,57,0,0,558,105,1,0,0,0,559,560,7,4,0,0,560,107,
        1,0,0,0,561,562,5,54,0,0,562,109,1,0,0,0,563,564,5,49,0,0,564,111,
        1,0,0,0,565,566,3,54,27,0,566,567,5,50,0,0,567,568,5,52,0,0,568,
        113,1,0,0,0,569,570,3,36,18,0,570,579,5,13,0,0,571,576,3,56,28,0,
        572,573,5,3,0,0,573,575,3,56,28,0,574,572,1,0,0,0,575,578,1,0,0,
        0,576,574,1,0,0,0,576,577,1,0,0,0,577,580,1,0,0,0,578,576,1,0,0,
        0,579,571,1,0,0,0,579,580,1,0,0,0,580,581,1,0,0,0,581,582,5,14,0,
        0,582,115,1,0,0,0,583,584,3,48,24,0,584,585,5,18,0,0,585,586,5,13,
        0,0,586,587,3,56,28,0,587,588,5,51,0,0,588,589,3,56,28,0,589,590,
        5,14,0,0,590,117,1,0,0,0,591,592,5,52,0,0,592,119,1,0,0,0,44,128,
        135,143,146,151,174,178,200,212,218,230,236,254,263,277,284,292,
        295,328,341,352,363,374,385,398,409,420,434,441,464,467,470,472,
        493,496,509,518,521,535,538,545,553,576,579
    ]

class BESSERActionLanguageParser ( Parser ):

    grammarFileName = "BESSERActionLanguage.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'def'", "'('", "','", "')'", "'->'", 
                     "':'", "'='", "';'", "'while'", "'do'", "'for'", "'in'", 
                     "'{'", "'}'", "'if'", "'else'", "'return'", "'[]'", 
                     "'?'", "'['", "']'", "'any'", "'float'", "'str'", "'int'", 
                     "'bool'", "'nothing'", "'||'", "'&&'", "'=='", "'!='", 
                     "'<'", "'<='", "'>'", "'>='", "'instanceof'", "'+'", 
                     "'-'", "'*'", "'/'", "'%'", "'!'", "'??'", "'.'", "'this'", 
                     "'new'", "'true'", "'false'", "'null'", "'::'", "'..'" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "ID", "INT", "FLOAT", "WS", "NL", "STR" ]

    RULE_function_definition = 0
    RULE_parameter = 1
    RULE_parameterWithDefault = 2
    RULE_statements = 3
    RULE_cond_loop = 4
    RULE_while = 5
    RULE_do_while = 6
    RULE_for = 7
    RULE_iterator = 8
    RULE_conditional_branch = 9
    RULE_block = 10
    RULE_condition = 11
    RULE_assign_target = 12
    RULE_explicit_declaration = 13
    RULE_assignment = 14
    RULE_return = 15
    RULE_type = 16
    RULE_single_type = 17
    RULE_sequence_type = 18
    RULE_optional_type = 19
    RULE_function_type = 20
    RULE_any_type = 21
    RULE_real_type = 22
    RULE_string_type = 23
    RULE_int_type = 24
    RULE_bool_type = 25
    RULE_nothing = 26
    RULE_classifier_type = 27
    RULE_expression = 28
    RULE_ternary = 29
    RULE_boolean = 30
    RULE_or = 31
    RULE_and = 32
    RULE_equality = 33
    RULE_comparison = 34
    RULE_instanceof = 35
    RULE_arithmetic = 36
    RULE_plus_minus = 37
    RULE_mult_div = 38
    RULE_remain = 39
    RULE_primary = 40
    RULE_selection_expression = 41
    RULE_field_access = 42
    RULE_array_access = 43
    RULE_function_call = 44
    RULE_atomic = 45
    RULE_procedure_call = 46
    RULE_this = 47
    RULE_new = 48
    RULE_literal = 49
    RULE_single_literal = 50
    RULE_int_literal = 51
    RULE_string_literal = 52
    RULE_bool_literal = 53
    RULE_real_literal = 54
    RULE_null_literal = 55
    RULE_enum_literal = 56
    RULE_sequence_literal = 57
    RULE_range_literal = 58
    RULE_symbol = 59

    ruleNames =  [ "function_definition", "parameter", "parameterWithDefault", 
                   "statements", "cond_loop", "while", "do_while", "for", 
                   "iterator", "conditional_branch", "block", "condition", 
                   "assign_target", "explicit_declaration", "assignment", 
                   "return", "type", "single_type", "sequence_type", "optional_type", 
                   "function_type", "any_type", "real_type", "string_type", 
                   "int_type", "bool_type", "nothing", "classifier_type", 
                   "expression", "ternary", "boolean", "or", "and", "equality", 
                   "comparison", "instanceof", "arithmetic", "plus_minus", 
                   "mult_div", "remain", "primary", "selection_expression", 
                   "field_access", "array_access", "function_call", "atomic", 
                   "procedure_call", "this", "new", "literal", "single_literal", 
                   "int_literal", "string_literal", "bool_literal", "real_literal", 
                   "null_literal", "enum_literal", "sequence_literal", "range_literal", 
                   "symbol" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    T__8=9
    T__9=10
    T__10=11
    T__11=12
    T__12=13
    T__13=14
    T__14=15
    T__15=16
    T__16=17
    T__17=18
    T__18=19
    T__19=20
    T__20=21
    T__21=22
    T__22=23
    T__23=24
    T__24=25
    T__25=26
    T__26=27
    T__27=28
    T__28=29
    T__29=30
    T__30=31
    T__31=32
    T__32=33
    T__33=34
    T__34=35
    T__35=36
    T__36=37
    T__37=38
    T__38=39
    T__39=40
    T__40=41
    T__41=42
    T__42=43
    T__43=44
    T__44=45
    T__45=46
    T__46=47
    T__47=48
    T__48=49
    T__49=50
    T__50=51
    ID=52
    INT=53
    FLOAT=54
    WS=55
    NL=56
    STR=57

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.2")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Function_definitionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.name = None # Token
            self._parameter = None # ParameterContext
            self.params = list() # of ParameterContexts
            self._parameterWithDefault = None # ParameterWithDefaultContext
            self.paramsDefault = list() # of ParameterWithDefaultContexts
            self.return_type = None # TypeContext
            self.body = None # BlockContext

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def block(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.BlockContext,0)


        def type_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.TypeContext,0)


        def parameter(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ParameterContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ParameterContext,i)


        def parameterWithDefault(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ParameterWithDefaultContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ParameterWithDefaultContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_function_definition

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFunction_definition" ):
                listener.enterFunction_definition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFunction_definition" ):
                listener.exitFunction_definition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFunction_definition" ):
                return visitor.visitFunction_definition(self)
            else:
                return visitor.visitChildren(self)




    def function_definition(self):

        localctx = BESSERActionLanguageParser.Function_definitionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_function_definition)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 120
            self.match(BESSERActionLanguageParser.T__0)
            self.state = 121
            localctx.name = self.match(BESSERActionLanguageParser.ID)
            self.state = 122
            self.match(BESSERActionLanguageParser.T__1)
            self.state = 146
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,3,self._ctx)
            if la_ == 1:
                self.state = 123
                localctx._parameter = self.parameter()
                localctx.params.append(localctx._parameter)
                self.state = 128
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,0,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 124
                        self.match(BESSERActionLanguageParser.T__2)
                        self.state = 125
                        localctx._parameter = self.parameter()
                        localctx.params.append(localctx._parameter) 
                    self.state = 130
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,0,self._ctx)

                self.state = 135
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==3:
                    self.state = 131
                    self.match(BESSERActionLanguageParser.T__2)
                    self.state = 132
                    localctx._parameterWithDefault = self.parameterWithDefault()
                    localctx.paramsDefault.append(localctx._parameterWithDefault)
                    self.state = 137
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)


            elif la_ == 2:
                self.state = 138
                localctx._parameterWithDefault = self.parameterWithDefault()
                localctx.paramsDefault.append(localctx._parameterWithDefault)
                self.state = 143
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==3:
                    self.state = 139
                    self.match(BESSERActionLanguageParser.T__2)
                    self.state = 140
                    localctx._parameterWithDefault = self.parameterWithDefault()
                    localctx.paramsDefault.append(localctx._parameterWithDefault)
                    self.state = 145
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 148
            self.match(BESSERActionLanguageParser.T__3)
            self.state = 151
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==5:
                self.state = 149
                self.match(BESSERActionLanguageParser.T__4)
                self.state = 150
                localctx.return_type = self.type_()


            self.state = 153
            localctx.body = self.block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParameterContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.name = None # Token
            self.declared_type = None # TypeContext

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def type_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.TypeContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_parameter

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterParameter" ):
                listener.enterParameter(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitParameter" ):
                listener.exitParameter(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitParameter" ):
                return visitor.visitParameter(self)
            else:
                return visitor.visitChildren(self)




    def parameter(self):

        localctx = BESSERActionLanguageParser.ParameterContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_parameter)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 155
            localctx.name = self.match(BESSERActionLanguageParser.ID)
            self.state = 156
            self.match(BESSERActionLanguageParser.T__5)
            self.state = 157
            localctx.declared_type = self.type_()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ParameterWithDefaultContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.name = None # Token
            self.declared_type = None # TypeContext
            self.expr = None # ExpressionContext

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def type_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.TypeContext,0)


        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_parameterWithDefault

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterParameterWithDefault" ):
                listener.enterParameterWithDefault(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitParameterWithDefault" ):
                listener.exitParameterWithDefault(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitParameterWithDefault" ):
                return visitor.visitParameterWithDefault(self)
            else:
                return visitor.visitChildren(self)




    def parameterWithDefault(self):

        localctx = BESSERActionLanguageParser.ParameterWithDefaultContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_parameterWithDefault)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 159
            localctx.name = self.match(BESSERActionLanguageParser.ID)
            self.state = 160
            self.match(BESSERActionLanguageParser.T__5)
            self.state = 161
            localctx.declared_type = self.type_()
            self.state = 162
            self.match(BESSERActionLanguageParser.T__6)
            self.state = 163
            localctx.expr = self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class StatementsContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def cond_loop(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Cond_loopContext,0)


        def for_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ForContext,0)


        def condition(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ConditionContext,0)


        def return_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ReturnContext,0)


        def function_definition(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Function_definitionContext,0)


        def assignment(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.AssignmentContext,0)


        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_statements

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterStatements" ):
                listener.enterStatements(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitStatements" ):
                listener.exitStatements(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitStatements" ):
                return visitor.visitStatements(self)
            else:
                return visitor.visitChildren(self)




    def statements(self):

        localctx = BESSERActionLanguageParser.StatementsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_statements)
        try:
            self.state = 174
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,5,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 165
                self.cond_loop()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 166
                self.for_()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 167
                self.condition()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 168
                self.return_()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 169
                self.function_definition()
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 170
                self.assignment()
                pass

            elif la_ == 7:
                self.enterOuterAlt(localctx, 7)
                self.state = 171
                self.expression()
                self.state = 172
                self.match(BESSERActionLanguageParser.T__7)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Cond_loopContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def while_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.WhileContext,0)


        def do_while(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Do_whileContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_cond_loop

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCond_loop" ):
                listener.enterCond_loop(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCond_loop" ):
                listener.exitCond_loop(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitCond_loop" ):
                return visitor.visitCond_loop(self)
            else:
                return visitor.visitChildren(self)




    def cond_loop(self):

        localctx = BESSERActionLanguageParser.Cond_loopContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_cond_loop)
        try:
            self.state = 178
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [9]:
                self.enterOuterAlt(localctx, 1)
                self.state = 176
                self.while_()
                pass
            elif token in [10]:
                self.enterOuterAlt(localctx, 2)
                self.state = 177
                self.do_while()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class WhileContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.cond = None # ExpressionContext
            self.body = None # BlockContext

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def block(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.BlockContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_while

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterWhile" ):
                listener.enterWhile(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitWhile" ):
                listener.exitWhile(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitWhile" ):
                return visitor.visitWhile(self)
            else:
                return visitor.visitChildren(self)




    def while_(self):

        localctx = BESSERActionLanguageParser.WhileContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_while)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 180
            self.match(BESSERActionLanguageParser.T__8)
            self.state = 181
            self.match(BESSERActionLanguageParser.T__1)
            self.state = 182
            localctx.cond = self.expression()
            self.state = 183
            self.match(BESSERActionLanguageParser.T__3)
            self.state = 184
            localctx.body = self.block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Do_whileContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.body = None # BlockContext
            self.cond = None # ExpressionContext

        def block(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.BlockContext,0)


        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_do_while

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDo_while" ):
                listener.enterDo_while(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDo_while" ):
                listener.exitDo_while(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitDo_while" ):
                return visitor.visitDo_while(self)
            else:
                return visitor.visitChildren(self)




    def do_while(self):

        localctx = BESSERActionLanguageParser.Do_whileContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_do_while)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 186
            self.match(BESSERActionLanguageParser.T__9)
            self.state = 187
            localctx.body = self.block()
            self.state = 188
            self.match(BESSERActionLanguageParser.T__8)
            self.state = 189
            self.match(BESSERActionLanguageParser.T__1)
            self.state = 190
            localctx.cond = self.expression()
            self.state = 191
            self.match(BESSERActionLanguageParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ForContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self._iterator = None # IteratorContext
            self.iterators = list() # of IteratorContexts
            self.body = None # BlockContext

        def iterator(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.IteratorContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.IteratorContext,i)


        def block(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.BlockContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_for

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFor" ):
                listener.enterFor(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFor" ):
                listener.exitFor(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFor" ):
                return visitor.visitFor(self)
            else:
                return visitor.visitChildren(self)




    def for_(self):

        localctx = BESSERActionLanguageParser.ForContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_for)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 193
            self.match(BESSERActionLanguageParser.T__10)
            self.state = 194
            self.match(BESSERActionLanguageParser.T__1)
            self.state = 195
            localctx._iterator = self.iterator()
            localctx.iterators.append(localctx._iterator)
            self.state = 200
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==3:
                self.state = 196
                self.match(BESSERActionLanguageParser.T__2)
                self.state = 197
                localctx._iterator = self.iterator()
                localctx.iterators.append(localctx._iterator)
                self.state = 202
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 203
            self.match(BESSERActionLanguageParser.T__3)
            self.state = 204
            localctx.body = self.block()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class IteratorContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.var_name = None # SymbolContext
            self.sequence = None # ExpressionContext

        def symbol(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.SymbolContext,0)


        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_iterator

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterIterator" ):
                listener.enterIterator(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitIterator" ):
                listener.exitIterator(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitIterator" ):
                return visitor.visitIterator(self)
            else:
                return visitor.visitChildren(self)




    def iterator(self):

        localctx = BESSERActionLanguageParser.IteratorContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_iterator)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 206
            localctx.var_name = self.symbol()
            self.state = 207
            self.match(BESSERActionLanguageParser.T__11)
            self.state = 208
            localctx.sequence = self.expression()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Conditional_branchContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def block(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.BlockContext,0)


        def condition(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ConditionContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_conditional_branch

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConditional_branch" ):
                listener.enterConditional_branch(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConditional_branch" ):
                listener.exitConditional_branch(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitConditional_branch" ):
                return visitor.visitConditional_branch(self)
            else:
                return visitor.visitChildren(self)




    def conditional_branch(self):

        localctx = BESSERActionLanguageParser.Conditional_branchContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_conditional_branch)
        try:
            self.state = 212
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [13]:
                self.enterOuterAlt(localctx, 1)
                self.state = 210
                self.block()
                pass
            elif token in [15]:
                self.enterOuterAlt(localctx, 2)
                self.state = 211
                self.condition()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class BlockContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self._statements = None # StatementsContext
            self.stmts = list() # of StatementsContexts

        def statements(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.StatementsContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.StatementsContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_block

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBlock" ):
                listener.enterBlock(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBlock" ):
                listener.exitBlock(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBlock" ):
                return visitor.visitBlock(self)
            else:
                return visitor.visitChildren(self)




    def block(self):

        localctx = BESSERActionLanguageParser.BlockContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_block)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 214
            self.match(BESSERActionLanguageParser.T__12)
            self.state = 218
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & 176735774191029766) != 0):
                self.state = 215
                localctx._statements = self.statements()
                localctx.stmts.append(localctx._statements)
                self.state = 220
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 221
            self.match(BESSERActionLanguageParser.T__13)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ConditionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.cond = None # ExpressionContext
            self.then = None # BlockContext
            self.elze = None # Conditional_branchContext

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def block(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.BlockContext,0)


        def conditional_branch(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Conditional_branchContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_condition

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCondition" ):
                listener.enterCondition(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCondition" ):
                listener.exitCondition(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitCondition" ):
                return visitor.visitCondition(self)
            else:
                return visitor.visitChildren(self)




    def condition(self):

        localctx = BESSERActionLanguageParser.ConditionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_condition)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 223
            self.match(BESSERActionLanguageParser.T__14)
            self.state = 224
            self.match(BESSERActionLanguageParser.T__1)
            self.state = 225
            localctx.cond = self.expression()
            self.state = 226
            self.match(BESSERActionLanguageParser.T__3)
            self.state = 227
            localctx.then = self.block()
            self.state = 230
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==16:
                self.state = 228
                self.match(BESSERActionLanguageParser.T__15)
                self.state = 229
                localctx.elze = self.conditional_branch()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Assign_targetContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def field_access(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Field_accessContext,0)


        def array_access(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Array_accessContext,0)


        def symbol(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.SymbolContext,0)


        def explicit_declaration(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Explicit_declarationContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_assign_target

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAssign_target" ):
                listener.enterAssign_target(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAssign_target" ):
                listener.exitAssign_target(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAssign_target" ):
                return visitor.visitAssign_target(self)
            else:
                return visitor.visitChildren(self)




    def assign_target(self):

        localctx = BESSERActionLanguageParser.Assign_targetContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_assign_target)
        try:
            self.state = 236
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,11,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 232
                self.field_access()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 233
                self.array_access()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 234
                self.symbol()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 235
                self.explicit_declaration()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Explicit_declarationContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.name = None # Token
            self.declared_type = None # TypeContext

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def type_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.TypeContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_explicit_declaration

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExplicit_declaration" ):
                listener.enterExplicit_declaration(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExplicit_declaration" ):
                listener.exitExplicit_declaration(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitExplicit_declaration" ):
                return visitor.visitExplicit_declaration(self)
            else:
                return visitor.visitChildren(self)




    def explicit_declaration(self):

        localctx = BESSERActionLanguageParser.Explicit_declarationContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_explicit_declaration)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 238
            localctx.name = self.match(BESSERActionLanguageParser.ID)
            self.state = 239
            self.match(BESSERActionLanguageParser.T__5)
            self.state = 240
            localctx.declared_type = self.type_()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AssignmentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.target = None # Assign_targetContext
            self.assignee = None # ExpressionContext

        def assign_target(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Assign_targetContext,0)


        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_assignment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAssignment" ):
                listener.enterAssignment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAssignment" ):
                listener.exitAssignment(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAssignment" ):
                return visitor.visitAssignment(self)
            else:
                return visitor.visitChildren(self)




    def assignment(self):

        localctx = BESSERActionLanguageParser.AssignmentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_assignment)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 242
            localctx.target = self.assign_target()
            self.state = 243
            self.match(BESSERActionLanguageParser.T__6)
            self.state = 244
            localctx.assignee = self.expression()
            self.state = 245
            self.match(BESSERActionLanguageParser.T__7)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ReturnContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.expr = None # ExpressionContext

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_return

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterReturn" ):
                listener.enterReturn(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitReturn" ):
                listener.exitReturn(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitReturn" ):
                return visitor.visitReturn(self)
            else:
                return visitor.visitChildren(self)




    def return_(self):

        localctx = BESSERActionLanguageParser.ReturnContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_return)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 247
            self.match(BESSERActionLanguageParser.T__16)
            self.state = 248
            localctx.expr = self.expression()
            self.state = 249
            self.match(BESSERActionLanguageParser.T__7)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def single_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Single_typeContext,0)


        def sequence_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Sequence_typeContext,0)


        def function_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Function_typeContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterType" ):
                listener.enterType(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitType" ):
                listener.exitType(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitType" ):
                return visitor.visitType(self)
            else:
                return visitor.visitChildren(self)




    def type_(self):

        localctx = BESSERActionLanguageParser.TypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_type)
        try:
            self.state = 254
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,12,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 251
                self.single_type()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 252
                self.sequence_type()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 253
                self.function_type()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Single_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def any_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Any_typeContext,0)


        def real_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Real_typeContext,0)


        def string_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.String_typeContext,0)


        def int_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Int_typeContext,0)


        def bool_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Bool_typeContext,0)


        def nothing(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.NothingContext,0)


        def classifier_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Classifier_typeContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_single_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingle_type" ):
                listener.enterSingle_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingle_type" ):
                listener.exitSingle_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSingle_type" ):
                return visitor.visitSingle_type(self)
            else:
                return visitor.visitChildren(self)




    def single_type(self):

        localctx = BESSERActionLanguageParser.Single_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_single_type)
        try:
            self.state = 263
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [22]:
                self.enterOuterAlt(localctx, 1)
                self.state = 256
                self.any_type()
                pass
            elif token in [23]:
                self.enterOuterAlt(localctx, 2)
                self.state = 257
                self.real_type()
                pass
            elif token in [24]:
                self.enterOuterAlt(localctx, 3)
                self.state = 258
                self.string_type()
                pass
            elif token in [25]:
                self.enterOuterAlt(localctx, 4)
                self.state = 259
                self.int_type()
                pass
            elif token in [26]:
                self.enterOuterAlt(localctx, 5)
                self.state = 260
                self.bool_type()
                pass
            elif token in [27]:
                self.enterOuterAlt(localctx, 6)
                self.state = 261
                self.nothing()
                pass
            elif token in [52]:
                self.enterOuterAlt(localctx, 7)
                self.state = 262
                self.classifier_type()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Sequence_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.the_type = None # Single_typeContext

        def single_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Single_typeContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_sequence_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSequence_type" ):
                listener.enterSequence_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSequence_type" ):
                listener.exitSequence_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSequence_type" ):
                return visitor.visitSequence_type(self)
            else:
                return visitor.visitChildren(self)




    def sequence_type(self):

        localctx = BESSERActionLanguageParser.Sequence_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_sequence_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 265
            localctx.the_type = self.single_type()
            self.state = 266
            self.match(BESSERActionLanguageParser.T__17)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Optional_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.the_type = None # TypeContext

        def type_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.TypeContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_optional_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOptional_type" ):
                listener.enterOptional_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOptional_type" ):
                listener.exitOptional_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitOptional_type" ):
                return visitor.visitOptional_type(self)
            else:
                return visitor.visitChildren(self)




    def optional_type(self):

        localctx = BESSERActionLanguageParser.Optional_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_optional_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 268
            localctx.the_type = self.type_()
            self.state = 269
            self.match(BESSERActionLanguageParser.T__18)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Function_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self._type = None # TypeContext
            self.params_type = list() # of TypeContexts
            self._optional_type = None # Optional_typeContext
            self.params_type_opt = list() # of Optional_typeContexts
            self.return_type = None # TypeContext

        def type_(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.TypeContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.TypeContext,i)


        def optional_type(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.Optional_typeContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.Optional_typeContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_function_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFunction_type" ):
                listener.enterFunction_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFunction_type" ):
                listener.exitFunction_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFunction_type" ):
                return visitor.visitFunction_type(self)
            else:
                return visitor.visitChildren(self)




    def function_type(self):

        localctx = BESSERActionLanguageParser.Function_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_function_type)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 271
            self.match(BESSERActionLanguageParser.T__19)
            self.state = 295
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,17,self._ctx)
            if la_ == 1:
                self.state = 272
                localctx._type = self.type_()
                localctx.params_type.append(localctx._type)
                self.state = 277
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,14,self._ctx)
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1:
                        self.state = 273
                        self.match(BESSERActionLanguageParser.T__2)
                        self.state = 274
                        localctx._type = self.type_()
                        localctx.params_type.append(localctx._type) 
                    self.state = 279
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,14,self._ctx)

                self.state = 284
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==3:
                    self.state = 280
                    self.match(BESSERActionLanguageParser.T__2)
                    self.state = 281
                    localctx._optional_type = self.optional_type()
                    localctx.params_type_opt.append(localctx._optional_type)
                    self.state = 286
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)


            elif la_ == 2:
                self.state = 287
                localctx._optional_type = self.optional_type()
                localctx.params_type_opt.append(localctx._optional_type)
                self.state = 292
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==3:
                    self.state = 288
                    self.match(BESSERActionLanguageParser.T__2)
                    self.state = 289
                    localctx._optional_type = self.optional_type()
                    localctx.params_type_opt.append(localctx._optional_type)
                    self.state = 294
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 297
            self.match(BESSERActionLanguageParser.T__20)
            self.state = 298
            self.match(BESSERActionLanguageParser.T__4)
            self.state = 299
            localctx.return_type = self.type_()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Any_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_any_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAny_type" ):
                listener.enterAny_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAny_type" ):
                listener.exitAny_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAny_type" ):
                return visitor.visitAny_type(self)
            else:
                return visitor.visitChildren(self)




    def any_type(self):

        localctx = BESSERActionLanguageParser.Any_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_any_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 301
            self.match(BESSERActionLanguageParser.T__21)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Real_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_real_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterReal_type" ):
                listener.enterReal_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitReal_type" ):
                listener.exitReal_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitReal_type" ):
                return visitor.visitReal_type(self)
            else:
                return visitor.visitChildren(self)




    def real_type(self):

        localctx = BESSERActionLanguageParser.Real_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_real_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 303
            self.match(BESSERActionLanguageParser.T__22)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class String_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_string_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterString_type" ):
                listener.enterString_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitString_type" ):
                listener.exitString_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitString_type" ):
                return visitor.visitString_type(self)
            else:
                return visitor.visitChildren(self)




    def string_type(self):

        localctx = BESSERActionLanguageParser.String_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_string_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 305
            self.match(BESSERActionLanguageParser.T__23)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Int_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_int_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInt_type" ):
                listener.enterInt_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInt_type" ):
                listener.exitInt_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInt_type" ):
                return visitor.visitInt_type(self)
            else:
                return visitor.visitChildren(self)




    def int_type(self):

        localctx = BESSERActionLanguageParser.Int_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_int_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 307
            self.match(BESSERActionLanguageParser.T__24)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Bool_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_bool_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBool_type" ):
                listener.enterBool_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBool_type" ):
                listener.exitBool_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBool_type" ):
                return visitor.visitBool_type(self)
            else:
                return visitor.visitChildren(self)




    def bool_type(self):

        localctx = BESSERActionLanguageParser.Bool_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_bool_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 309
            self.match(BESSERActionLanguageParser.T__25)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NothingContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_nothing

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNothing" ):
                listener.enterNothing(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNothing" ):
                listener.exitNothing(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNothing" ):
                return visitor.visitNothing(self)
            else:
                return visitor.visitChildren(self)




    def nothing(self):

        localctx = BESSERActionLanguageParser.NothingContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_nothing)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 311
            self.match(BESSERActionLanguageParser.T__26)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Classifier_typeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.name = None # Token

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_classifier_type

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterClassifier_type" ):
                listener.enterClassifier_type(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitClassifier_type" ):
                listener.exitClassifier_type(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitClassifier_type" ):
                return visitor.visitClassifier_type(self)
            else:
                return visitor.visitChildren(self)




    def classifier_type(self):

        localctx = BESSERActionLanguageParser.Classifier_typeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_classifier_type)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 313
            localctx.name = self.match(BESSERActionLanguageParser.ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ExpressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ternary(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.TernaryContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_expression

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterExpression" ):
                listener.enterExpression(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitExpression" ):
                listener.exitExpression(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitExpression" ):
                return visitor.visitExpression(self)
            else:
                return visitor.visitChildren(self)




    def expression(self):

        localctx = BESSERActionLanguageParser.ExpressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_expression)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 315
            self.ternary(0)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TernaryContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.expr = None # TernaryContext
            self.then = None # ExpressionContext
            self.elze = None # ExpressionContext

        def boolean(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.BooleanContext,0)


        def ternary(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.TernaryContext,0)


        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_ternary

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTernary" ):
                listener.enterTernary(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTernary" ):
                listener.exitTernary(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitTernary" ):
                return visitor.visitTernary(self)
            else:
                return visitor.visitChildren(self)



    def ternary(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.TernaryContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 58
        self.enterRecursionRule(localctx, 58, self.RULE_ternary, _p)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 318
            self.boolean()
            self._ctx.stop = self._input.LT(-1)
            self.state = 328
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,18,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.TernaryContext(self, _parentctx, _parentState)
                    localctx.expr = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_ternary)
                    self.state = 320
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 321
                    self.match(BESSERActionLanguageParser.T__18)
                    self.state = 322
                    localctx.then = self.expression()
                    self.state = 323
                    self.match(BESSERActionLanguageParser.T__5)
                    self.state = 324
                    localctx.elze = self.expression() 
                self.state = 330
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,18,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class BooleanContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def or_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.OrContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_boolean

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBoolean" ):
                listener.enterBoolean(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBoolean" ):
                listener.exitBoolean(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBoolean" ):
                return visitor.visitBoolean(self)
            else:
                return visitor.visitChildren(self)




    def boolean(self):

        localctx = BESSERActionLanguageParser.BooleanContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_boolean)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 331
            self.or_(0)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class OrContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.left = None # OrContext
            self.right = None # AndContext

        def and_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.AndContext,0)


        def or_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.OrContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_or

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOr" ):
                listener.enterOr(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOr" ):
                listener.exitOr(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitOr" ):
                return visitor.visitOr(self)
            else:
                return visitor.visitChildren(self)



    def or_(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.OrContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 62
        self.enterRecursionRule(localctx, 62, self.RULE_or, _p)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 334
            self.and_(0)
            self._ctx.stop = self._input.LT(-1)
            self.state = 341
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,19,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.OrContext(self, _parentctx, _parentState)
                    localctx.left = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_or)
                    self.state = 336
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 337
                    self.match(BESSERActionLanguageParser.T__27)
                    self.state = 338
                    localctx.right = self.and_(0) 
                self.state = 343
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,19,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class AndContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.left = None # AndContext
            self.right = None # EqualityContext

        def equality(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.EqualityContext,0)


        def and_(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.AndContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_and

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAnd" ):
                listener.enterAnd(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAnd" ):
                listener.exitAnd(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAnd" ):
                return visitor.visitAnd(self)
            else:
                return visitor.visitChildren(self)



    def and_(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.AndContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 64
        self.enterRecursionRule(localctx, 64, self.RULE_and, _p)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 345
            self.equality(0)
            self._ctx.stop = self._input.LT(-1)
            self.state = 352
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,20,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.AndContext(self, _parentctx, _parentState)
                    localctx.left = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_and)
                    self.state = 347
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 348
                    self.match(BESSERActionLanguageParser.T__28)
                    self.state = 349
                    localctx.right = self.equality(0) 
                self.state = 354
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,20,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class EqualityContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.left = None # EqualityContext
            self.op = None # Token
            self.right = None # ComparisonContext

        def comparison(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ComparisonContext,0)


        def equality(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.EqualityContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_equality

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEquality" ):
                listener.enterEquality(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEquality" ):
                listener.exitEquality(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEquality" ):
                return visitor.visitEquality(self)
            else:
                return visitor.visitChildren(self)



    def equality(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.EqualityContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 66
        self.enterRecursionRule(localctx, 66, self.RULE_equality, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 356
            self.comparison(0)
            self._ctx.stop = self._input.LT(-1)
            self.state = 363
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,21,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.EqualityContext(self, _parentctx, _parentState)
                    localctx.left = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_equality)
                    self.state = 358
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 359
                    localctx.op = self._input.LT(1)
                    _la = self._input.LA(1)
                    if not(_la==30 or _la==31):
                        localctx.op = self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 360
                    localctx.right = self.comparison(0) 
                self.state = 365
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,21,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class ComparisonContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.left = None # ComparisonContext
            self.op = None # Token
            self.right = None # InstanceofContext

        def instanceof(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.InstanceofContext,0)


        def comparison(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ComparisonContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_comparison

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComparison" ):
                listener.enterComparison(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComparison" ):
                listener.exitComparison(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitComparison" ):
                return visitor.visitComparison(self)
            else:
                return visitor.visitChildren(self)



    def comparison(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.ComparisonContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 68
        self.enterRecursionRule(localctx, 68, self.RULE_comparison, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 367
            self.instanceof(0)
            self._ctx.stop = self._input.LT(-1)
            self.state = 374
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,22,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.ComparisonContext(self, _parentctx, _parentState)
                    localctx.left = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_comparison)
                    self.state = 369
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 370
                    localctx.op = self._input.LT(1)
                    _la = self._input.LA(1)
                    if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 64424509440) != 0)):
                        localctx.op = self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 371
                    localctx.right = self.instanceof(0) 
                self.state = 376
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,22,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class InstanceofContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.instance = None # InstanceofContext
            self.the_type = None # Classifier_typeContext

        def arithmetic(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ArithmeticContext,0)


        def instanceof(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.InstanceofContext,0)


        def classifier_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Classifier_typeContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_instanceof

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInstanceof" ):
                listener.enterInstanceof(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInstanceof" ):
                listener.exitInstanceof(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInstanceof" ):
                return visitor.visitInstanceof(self)
            else:
                return visitor.visitChildren(self)



    def instanceof(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.InstanceofContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 70
        self.enterRecursionRule(localctx, 70, self.RULE_instanceof, _p)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 378
            self.arithmetic()
            self._ctx.stop = self._input.LT(-1)
            self.state = 385
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,23,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.InstanceofContext(self, _parentctx, _parentState)
                    localctx.instance = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_instanceof)
                    self.state = 380
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 381
                    self.match(BESSERActionLanguageParser.T__35)
                    self.state = 382
                    localctx.the_type = self.classifier_type() 
                self.state = 387
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,23,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class ArithmeticContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def plus_minus(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Plus_minusContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_arithmetic

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArithmetic" ):
                listener.enterArithmetic(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArithmetic" ):
                listener.exitArithmetic(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitArithmetic" ):
                return visitor.visitArithmetic(self)
            else:
                return visitor.visitChildren(self)




    def arithmetic(self):

        localctx = BESSERActionLanguageParser.ArithmeticContext(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_arithmetic)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 388
            self.plus_minus(0)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Plus_minusContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.left = None # Plus_minusContext
            self.op = None # Token
            self.right = None # Mult_divContext

        def mult_div(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Mult_divContext,0)


        def plus_minus(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Plus_minusContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_plus_minus

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPlus_minus" ):
                listener.enterPlus_minus(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPlus_minus" ):
                listener.exitPlus_minus(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitPlus_minus" ):
                return visitor.visitPlus_minus(self)
            else:
                return visitor.visitChildren(self)



    def plus_minus(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.Plus_minusContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 74
        self.enterRecursionRule(localctx, 74, self.RULE_plus_minus, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 391
            self.mult_div(0)
            self._ctx.stop = self._input.LT(-1)
            self.state = 398
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,24,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.Plus_minusContext(self, _parentctx, _parentState)
                    localctx.left = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_plus_minus)
                    self.state = 393
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 394
                    localctx.op = self._input.LT(1)
                    _la = self._input.LA(1)
                    if not(_la==37 or _la==38):
                        localctx.op = self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 395
                    localctx.right = self.mult_div(0) 
                self.state = 400
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,24,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class Mult_divContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.left = None # Mult_divContext
            self.op = None # Token
            self.right = None # RemainContext

        def remain(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.RemainContext,0)


        def mult_div(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Mult_divContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_mult_div

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMult_div" ):
                listener.enterMult_div(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMult_div" ):
                listener.exitMult_div(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitMult_div" ):
                return visitor.visitMult_div(self)
            else:
                return visitor.visitChildren(self)



    def mult_div(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.Mult_divContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 76
        self.enterRecursionRule(localctx, 76, self.RULE_mult_div, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 402
            self.remain(0)
            self._ctx.stop = self._input.LT(-1)
            self.state = 409
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,25,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.Mult_divContext(self, _parentctx, _parentState)
                    localctx.left = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_mult_div)
                    self.state = 404
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 405
                    localctx.op = self._input.LT(1)
                    _la = self._input.LA(1)
                    if not(_la==39 or _la==40):
                        localctx.op = self._errHandler.recoverInline(self)
                    else:
                        self._errHandler.reportMatch(self)
                        self.consume()
                    self.state = 406
                    localctx.right = self.remain(0) 
                self.state = 411
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,25,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class RemainContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.left = None # RemainContext
            self.right = None # PrimaryContext

        def primary(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.PrimaryContext,0)


        def remain(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.RemainContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_remain

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRemain" ):
                listener.enterRemain(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRemain" ):
                listener.exitRemain(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitRemain" ):
                return visitor.visitRemain(self)
            else:
                return visitor.visitChildren(self)



    def remain(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.RemainContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 78
        self.enterRecursionRule(localctx, 78, self.RULE_remain, _p)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 413
            self.primary(0)
            self._ctx.stop = self._input.LT(-1)
            self.state = 420
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,26,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.RemainContext(self, _parentctx, _parentState)
                    localctx.left = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_remain)
                    self.state = 415
                    if not self.precpred(self._ctx, 2):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                    self.state = 416
                    self.match(BESSERActionLanguageParser.T__40)
                    self.state = 417
                    localctx.right = self.primary(0) 
                self.state = 422
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,26,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class PrimaryContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_primary

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)


    class CastContext(PrimaryContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.PrimaryContext
            super().__init__(parser)
            self.as_ = None # Classifier_typeContext
            self.expr = None # ExpressionContext
            self.copyFrom(ctx)

        def classifier_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Classifier_typeContext,0)

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCast" ):
                listener.enterCast(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCast" ):
                listener.exitCast(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitCast" ):
                return visitor.visitCast(self)
            else:
                return visitor.visitChildren(self)


    class NotContext(PrimaryContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.PrimaryContext
            super().__init__(parser)
            self.expr = None # ExpressionContext
            self.copyFrom(ctx)

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNot" ):
                listener.enterNot(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNot" ):
                listener.exitNot(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNot" ):
                return visitor.visitNot(self)
            else:
                return visitor.visitChildren(self)


    class NullCoalessingContext(PrimaryContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.PrimaryContext
            super().__init__(parser)
            self.nullable = None # PrimaryContext
            self.elze = None # ExpressionContext
            self.copyFrom(ctx)

        def primary(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.PrimaryContext,0)

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNullCoalessing" ):
                listener.enterNullCoalessing(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNullCoalessing" ):
                listener.exitNullCoalessing(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNullCoalessing" ):
                return visitor.visitNullCoalessing(self)
            else:
                return visitor.visitChildren(self)


    class SelectContext(PrimaryContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.PrimaryContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def selection_expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Selection_expressionContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSelect" ):
                listener.enterSelect(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSelect" ):
                listener.exitSelect(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSelect" ):
                return visitor.visitSelect(self)
            else:
                return visitor.visitChildren(self)


    class MinusContext(PrimaryContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.PrimaryContext
            super().__init__(parser)
            self.expr = None # ExpressionContext
            self.copyFrom(ctx)

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterMinus" ):
                listener.enterMinus(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitMinus" ):
                listener.exitMinus(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitMinus" ):
                return visitor.visitMinus(self)
            else:
                return visitor.visitChildren(self)



    def primary(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.PrimaryContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 80
        self.enterRecursionRule(localctx, 80, self.RULE_primary, _p)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 434
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,27,self._ctx)
            if la_ == 1:
                localctx = BESSERActionLanguageParser.SelectContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx

                self.state = 424
                self.selection_expression(0)
                pass

            elif la_ == 2:
                localctx = BESSERActionLanguageParser.MinusContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 425
                self.match(BESSERActionLanguageParser.T__37)
                self.state = 426
                localctx.expr = self.expression()
                pass

            elif la_ == 3:
                localctx = BESSERActionLanguageParser.NotContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 427
                self.match(BESSERActionLanguageParser.T__41)
                self.state = 428
                localctx.expr = self.expression()
                pass

            elif la_ == 4:
                localctx = BESSERActionLanguageParser.CastContext(self, localctx)
                self._ctx = localctx
                _prevctx = localctx
                self.state = 429
                self.match(BESSERActionLanguageParser.T__1)
                self.state = 430
                localctx.as_ = self.classifier_type()
                self.state = 431
                self.match(BESSERActionLanguageParser.T__3)
                self.state = 432
                localctx.expr = self.expression()
                pass


            self._ctx.stop = self._input.LT(-1)
            self.state = 441
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,28,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    localctx = BESSERActionLanguageParser.NullCoalessingContext(self, BESSERActionLanguageParser.PrimaryContext(self, _parentctx, _parentState))
                    localctx.nullable = _prevctx
                    self.pushNewRecursionContext(localctx, _startState, self.RULE_primary)
                    self.state = 436
                    if not self.precpred(self._ctx, 1):
                        from antlr4.error.Errors import FailedPredicateException
                        raise FailedPredicateException(self, "self.precpred(self._ctx, 1)")
                    self.state = 437
                    self.match(BESSERActionLanguageParser.T__42)
                    self.state = 438
                    localctx.elze = self.expression() 
                self.state = 443
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,28,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class Selection_expressionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_selection_expression

     
        def copyFrom(self, ctx:ParserRuleContext):
            super().copyFrom(ctx)


    class ArrayAccessContext(Selection_expressionContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.Selection_expressionContext
            super().__init__(parser)
            self.receiver = None # Selection_expressionContext
            self.index = None # ExpressionContext
            self.copyFrom(ctx)

        def selection_expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Selection_expressionContext,0)

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArrayAccess" ):
                listener.enterArrayAccess(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArrayAccess" ):
                listener.exitArrayAccess(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitArrayAccess" ):
                return visitor.visitArrayAccess(self)
            else:
                return visitor.visitChildren(self)


    class FunctionCallContext(Selection_expressionContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.Selection_expressionContext
            super().__init__(parser)
            self.receiver = None # Selection_expressionContext
            self.name = None # Token
            self._expression = None # ExpressionContext
            self.args = list() # of ExpressionContexts
            self.copyFrom(ctx)

        def selection_expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Selection_expressionContext,0)

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)
        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,i)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFunctionCall" ):
                listener.enterFunctionCall(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFunctionCall" ):
                listener.exitFunctionCall(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFunctionCall" ):
                return visitor.visitFunctionCall(self)
            else:
                return visitor.visitChildren(self)


    class FieldAccessContext(Selection_expressionContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.Selection_expressionContext
            super().__init__(parser)
            self.receiver = None # Selection_expressionContext
            self.field = None # Token
            self.copyFrom(ctx)

        def selection_expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Selection_expressionContext,0)

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFieldAccess" ):
                listener.enterFieldAccess(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFieldAccess" ):
                listener.exitFieldAccess(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFieldAccess" ):
                return visitor.visitFieldAccess(self)
            else:
                return visitor.visitChildren(self)


    class AtomContext(Selection_expressionContext):

        def __init__(self, parser, ctx:ParserRuleContext): # actually a BESSERActionLanguageParser.Selection_expressionContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def atomic(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.AtomicContext,0)


        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtom" ):
                listener.enterAtom(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtom" ):
                listener.exitAtom(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAtom" ):
                return visitor.visitAtom(self)
            else:
                return visitor.visitChildren(self)



    def selection_expression(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = BESSERActionLanguageParser.Selection_expressionContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 82
        self.enterRecursionRule(localctx, 82, self.RULE_selection_expression, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            localctx = BESSERActionLanguageParser.AtomContext(self, localctx)
            self._ctx = localctx
            _prevctx = localctx

            self.state = 445
            self.atomic()
            self._ctx.stop = self._input.LT(-1)
            self.state = 472
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,32,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    self.state = 470
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,31,self._ctx)
                    if la_ == 1:
                        localctx = BESSERActionLanguageParser.FieldAccessContext(self, BESSERActionLanguageParser.Selection_expressionContext(self, _parentctx, _parentState))
                        localctx.receiver = _prevctx
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_selection_expression)
                        self.state = 447
                        if not self.precpred(self._ctx, 3):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 3)")
                        self.state = 448
                        self.match(BESSERActionLanguageParser.T__43)
                        self.state = 449
                        localctx.field = self.match(BESSERActionLanguageParser.ID)
                        pass

                    elif la_ == 2:
                        localctx = BESSERActionLanguageParser.ArrayAccessContext(self, BESSERActionLanguageParser.Selection_expressionContext(self, _parentctx, _parentState))
                        localctx.receiver = _prevctx
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_selection_expression)
                        self.state = 450
                        if not self.precpred(self._ctx, 2):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 2)")
                        self.state = 451
                        self.match(BESSERActionLanguageParser.T__19)
                        self.state = 452
                        localctx.index = self.expression()
                        self.state = 453
                        self.match(BESSERActionLanguageParser.T__20)
                        pass

                    elif la_ == 3:
                        localctx = BESSERActionLanguageParser.FunctionCallContext(self, BESSERActionLanguageParser.Selection_expressionContext(self, _parentctx, _parentState))
                        localctx.receiver = _prevctx
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_selection_expression)
                        self.state = 455
                        if not self.precpred(self._ctx, 1):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 1)")
                        self.state = 456
                        self.match(BESSERActionLanguageParser.T__43)
                        self.state = 457
                        localctx.name = self.match(BESSERActionLanguageParser.ID)
                        self.state = 458
                        self.match(BESSERActionLanguageParser.T__1)
                        self.state = 467
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        if (((_la) & ~0x3f) == 0 and ((1 << _la) & 176735774190862340) != 0):
                            self.state = 459
                            localctx._expression = self.expression()
                            localctx.args.append(localctx._expression)
                            self.state = 464
                            self._errHandler.sync(self)
                            _la = self._input.LA(1)
                            while _la==3:
                                self.state = 460
                                self.match(BESSERActionLanguageParser.T__2)
                                self.state = 461
                                localctx._expression = self.expression()
                                localctx.args.append(localctx._expression)
                                self.state = 466
                                self._errHandler.sync(self)
                                _la = self._input.LA(1)



                        self.state = 469
                        self.match(BESSERActionLanguageParser.T__3)
                        pass

             
                self.state = 474
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,32,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class Field_accessContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.receiver = None # Selection_expressionContext
            self.field = None # Token

        def selection_expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Selection_expressionContext,0)


        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_field_access

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterField_access" ):
                listener.enterField_access(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitField_access" ):
                listener.exitField_access(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitField_access" ):
                return visitor.visitField_access(self)
            else:
                return visitor.visitChildren(self)




    def field_access(self):

        localctx = BESSERActionLanguageParser.Field_accessContext(self, self._ctx, self.state)
        self.enterRule(localctx, 84, self.RULE_field_access)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 475
            localctx.receiver = self.selection_expression(0)
            self.state = 476
            self.match(BESSERActionLanguageParser.T__43)
            self.state = 477
            localctx.field = self.match(BESSERActionLanguageParser.ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Array_accessContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.receiver = None # Selection_expressionContext
            self.index = None # ExpressionContext

        def selection_expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Selection_expressionContext,0)


        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_array_access

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArray_access" ):
                listener.enterArray_access(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArray_access" ):
                listener.exitArray_access(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitArray_access" ):
                return visitor.visitArray_access(self)
            else:
                return visitor.visitChildren(self)




    def array_access(self):

        localctx = BESSERActionLanguageParser.Array_accessContext(self, self._ctx, self.state)
        self.enterRule(localctx, 86, self.RULE_array_access)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 479
            localctx.receiver = self.selection_expression(0)
            self.state = 480
            self.match(BESSERActionLanguageParser.T__19)
            self.state = 481
            localctx.index = self.expression()
            self.state = 482
            self.match(BESSERActionLanguageParser.T__20)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Function_callContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.receiver = None # Selection_expressionContext
            self.name = None # Token
            self._expression = None # ExpressionContext
            self.args = list() # of ExpressionContexts

        def selection_expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Selection_expressionContext,0)


        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_function_call

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFunction_call" ):
                listener.enterFunction_call(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFunction_call" ):
                listener.exitFunction_call(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitFunction_call" ):
                return visitor.visitFunction_call(self)
            else:
                return visitor.visitChildren(self)




    def function_call(self):

        localctx = BESSERActionLanguageParser.Function_callContext(self, self._ctx, self.state)
        self.enterRule(localctx, 88, self.RULE_function_call)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 484
            localctx.receiver = self.selection_expression(0)
            self.state = 485
            self.match(BESSERActionLanguageParser.T__43)
            self.state = 486
            localctx.name = self.match(BESSERActionLanguageParser.ID)
            self.state = 487
            self.match(BESSERActionLanguageParser.T__1)
            self.state = 496
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 176735774190862340) != 0):
                self.state = 488
                localctx._expression = self.expression()
                localctx.args.append(localctx._expression)
                self.state = 493
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==3:
                    self.state = 489
                    self.match(BESSERActionLanguageParser.T__2)
                    self.state = 490
                    localctx._expression = self.expression()
                    localctx.args.append(localctx._expression)
                    self.state = 495
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 498
            self.match(BESSERActionLanguageParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AtomicContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.expr = None # ExpressionContext

        def expression(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,0)


        def procedure_call(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Procedure_callContext,0)


        def literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.LiteralContext,0)


        def this(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.ThisContext,0)


        def new(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.NewContext,0)


        def symbol(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.SymbolContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_atomic

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtomic" ):
                listener.enterAtomic(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtomic" ):
                listener.exitAtomic(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAtomic" ):
                return visitor.visitAtomic(self)
            else:
                return visitor.visitChildren(self)




    def atomic(self):

        localctx = BESSERActionLanguageParser.AtomicContext(self, self._ctx, self.state)
        self.enterRule(localctx, 90, self.RULE_atomic)
        try:
            self.state = 509
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,35,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 500
                self.match(BESSERActionLanguageParser.T__1)
                self.state = 501
                localctx.expr = self.expression()
                self.state = 502
                self.match(BESSERActionLanguageParser.T__3)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 504
                self.procedure_call()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 505
                self.literal()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 506
                self.this()
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 507
                self.new()
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 508
                self.symbol()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Procedure_callContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.name = None # Token
            self._expression = None # ExpressionContext
            self.args = list() # of ExpressionContexts

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_procedure_call

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProcedure_call" ):
                listener.enterProcedure_call(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProcedure_call" ):
                listener.exitProcedure_call(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitProcedure_call" ):
                return visitor.visitProcedure_call(self)
            else:
                return visitor.visitChildren(self)




    def procedure_call(self):

        localctx = BESSERActionLanguageParser.Procedure_callContext(self, self._ctx, self.state)
        self.enterRule(localctx, 92, self.RULE_procedure_call)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 511
            localctx.name = self.match(BESSERActionLanguageParser.ID)
            self.state = 512
            self.match(BESSERActionLanguageParser.T__1)
            self.state = 521
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 176735774190862340) != 0):
                self.state = 513
                localctx._expression = self.expression()
                localctx.args.append(localctx._expression)
                self.state = 518
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==3:
                    self.state = 514
                    self.match(BESSERActionLanguageParser.T__2)
                    self.state = 515
                    localctx._expression = self.expression()
                    localctx.args.append(localctx._expression)
                    self.state = 520
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 523
            self.match(BESSERActionLanguageParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ThisContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_this

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterThis" ):
                listener.enterThis(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitThis" ):
                listener.exitThis(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitThis" ):
                return visitor.visitThis(self)
            else:
                return visitor.visitChildren(self)




    def this(self):

        localctx = BESSERActionLanguageParser.ThisContext(self, self._ctx, self.state)
        self.enterRule(localctx, 94, self.RULE_this)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 525
            self.match(BESSERActionLanguageParser.T__44)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NewContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.clazz = None # Classifier_typeContext
            self._expression = None # ExpressionContext
            self.args = list() # of ExpressionContexts

        def classifier_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Classifier_typeContext,0)


        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_new

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNew" ):
                listener.enterNew(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNew" ):
                listener.exitNew(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNew" ):
                return visitor.visitNew(self)
            else:
                return visitor.visitChildren(self)




    def new(self):

        localctx = BESSERActionLanguageParser.NewContext(self, self._ctx, self.state)
        self.enterRule(localctx, 96, self.RULE_new)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 527
            self.match(BESSERActionLanguageParser.T__45)
            self.state = 528
            localctx.clazz = self.classifier_type()
            self.state = 529
            self.match(BESSERActionLanguageParser.T__1)
            self.state = 538
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 176735774190862340) != 0):
                self.state = 530
                localctx._expression = self.expression()
                localctx.args.append(localctx._expression)
                self.state = 535
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==3:
                    self.state = 531
                    self.match(BESSERActionLanguageParser.T__2)
                    self.state = 532
                    localctx._expression = self.expression()
                    localctx.args.append(localctx._expression)
                    self.state = 537
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 540
            self.match(BESSERActionLanguageParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LiteralContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def single_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Single_literalContext,0)


        def sequence_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Sequence_literalContext,0)


        def range_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Range_literalContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLiteral" ):
                listener.enterLiteral(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLiteral" ):
                listener.exitLiteral(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLiteral" ):
                return visitor.visitLiteral(self)
            else:
                return visitor.visitChildren(self)




    def literal(self):

        localctx = BESSERActionLanguageParser.LiteralContext(self, self._ctx, self.state)
        self.enterRule(localctx, 98, self.RULE_literal)
        try:
            self.state = 545
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,40,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 542
                self.single_literal()
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 543
                self.sequence_literal()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 544
                self.range_literal()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Single_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def int_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Int_literalContext,0)


        def string_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.String_literalContext,0)


        def bool_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Bool_literalContext,0)


        def real_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Real_literalContext,0)


        def null_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Null_literalContext,0)


        def enum_literal(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Enum_literalContext,0)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_single_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSingle_literal" ):
                listener.enterSingle_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSingle_literal" ):
                listener.exitSingle_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSingle_literal" ):
                return visitor.visitSingle_literal(self)
            else:
                return visitor.visitChildren(self)




    def single_literal(self):

        localctx = BESSERActionLanguageParser.Single_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 100, self.RULE_single_literal)
        try:
            self.state = 553
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [53]:
                self.enterOuterAlt(localctx, 1)
                self.state = 547
                self.int_literal()
                pass
            elif token in [57]:
                self.enterOuterAlt(localctx, 2)
                self.state = 548
                self.string_literal()
                pass
            elif token in [47, 48]:
                self.enterOuterAlt(localctx, 3)
                self.state = 549
                self.bool_literal()
                pass
            elif token in [54]:
                self.enterOuterAlt(localctx, 4)
                self.state = 550
                self.real_literal()
                pass
            elif token in [49]:
                self.enterOuterAlt(localctx, 5)
                self.state = 551
                self.null_literal()
                pass
            elif token in [52]:
                self.enterOuterAlt(localctx, 6)
                self.state = 552
                self.enum_literal()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Int_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.value = None # Token

        def INT(self):
            return self.getToken(BESSERActionLanguageParser.INT, 0)

        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_int_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterInt_literal" ):
                listener.enterInt_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitInt_literal" ):
                listener.exitInt_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitInt_literal" ):
                return visitor.visitInt_literal(self)
            else:
                return visitor.visitChildren(self)




    def int_literal(self):

        localctx = BESSERActionLanguageParser.Int_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 102, self.RULE_int_literal)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 555
            localctx.value = self.match(BESSERActionLanguageParser.INT)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class String_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.value = None # Token

        def STR(self):
            return self.getToken(BESSERActionLanguageParser.STR, 0)

        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_string_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterString_literal" ):
                listener.enterString_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitString_literal" ):
                listener.exitString_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitString_literal" ):
                return visitor.visitString_literal(self)
            else:
                return visitor.visitChildren(self)




    def string_literal(self):

        localctx = BESSERActionLanguageParser.String_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 104, self.RULE_string_literal)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 557
            localctx.value = self.match(BESSERActionLanguageParser.STR)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Bool_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.value = None # Token


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_bool_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterBool_literal" ):
                listener.enterBool_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitBool_literal" ):
                listener.exitBool_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitBool_literal" ):
                return visitor.visitBool_literal(self)
            else:
                return visitor.visitChildren(self)




    def bool_literal(self):

        localctx = BESSERActionLanguageParser.Bool_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 106, self.RULE_bool_literal)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 559
            localctx.value = self._input.LT(1)
            _la = self._input.LA(1)
            if not(_la==47 or _la==48):
                localctx.value = self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Real_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.value = None # Token

        def FLOAT(self):
            return self.getToken(BESSERActionLanguageParser.FLOAT, 0)

        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_real_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterReal_literal" ):
                listener.enterReal_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitReal_literal" ):
                listener.exitReal_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitReal_literal" ):
                return visitor.visitReal_literal(self)
            else:
                return visitor.visitChildren(self)




    def real_literal(self):

        localctx = BESSERActionLanguageParser.Real_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 108, self.RULE_real_literal)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 561
            localctx.value = self.match(BESSERActionLanguageParser.FLOAT)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Null_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.value = None # Token


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_null_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNull_literal" ):
                listener.enterNull_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNull_literal" ):
                listener.exitNull_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitNull_literal" ):
                return visitor.visitNull_literal(self)
            else:
                return visitor.visitChildren(self)




    def null_literal(self):

        localctx = BESSERActionLanguageParser.Null_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 110, self.RULE_null_literal)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 563
            localctx.value = self.match(BESSERActionLanguageParser.T__48)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Enum_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.enum = None # Classifier_typeContext
            self.name = None # Token

        def classifier_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Classifier_typeContext,0)


        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_enum_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEnum_literal" ):
                listener.enterEnum_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEnum_literal" ):
                listener.exitEnum_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEnum_literal" ):
                return visitor.visitEnum_literal(self)
            else:
                return visitor.visitChildren(self)




    def enum_literal(self):

        localctx = BESSERActionLanguageParser.Enum_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 112, self.RULE_enum_literal)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 565
            localctx.enum = self.classifier_type()
            self.state = 566
            self.match(BESSERActionLanguageParser.T__49)
            self.state = 567
            localctx.name = self.match(BESSERActionLanguageParser.ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Sequence_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.the_type = None # Sequence_typeContext
            self._expression = None # ExpressionContext
            self.values = list() # of ExpressionContexts

        def sequence_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Sequence_typeContext,0)


        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_sequence_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSequence_literal" ):
                listener.enterSequence_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSequence_literal" ):
                listener.exitSequence_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSequence_literal" ):
                return visitor.visitSequence_literal(self)
            else:
                return visitor.visitChildren(self)




    def sequence_literal(self):

        localctx = BESSERActionLanguageParser.Sequence_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 114, self.RULE_sequence_literal)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 569
            localctx.the_type = self.sequence_type()
            self.state = 570
            self.match(BESSERActionLanguageParser.T__12)
            self.state = 579
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 176735774190862340) != 0):
                self.state = 571
                localctx._expression = self.expression()
                localctx.values.append(localctx._expression)
                self.state = 576
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==3:
                    self.state = 572
                    self.match(BESSERActionLanguageParser.T__2)
                    self.state = 573
                    localctx._expression = self.expression()
                    localctx.values.append(localctx._expression)
                    self.state = 578
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)



            self.state = 581
            self.match(BESSERActionLanguageParser.T__13)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Range_literalContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.the_type = None # Int_typeContext
            self.first = None # ExpressionContext
            self.last = None # ExpressionContext

        def int_type(self):
            return self.getTypedRuleContext(BESSERActionLanguageParser.Int_typeContext,0)


        def expression(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(BESSERActionLanguageParser.ExpressionContext)
            else:
                return self.getTypedRuleContext(BESSERActionLanguageParser.ExpressionContext,i)


        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_range_literal

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRange_literal" ):
                listener.enterRange_literal(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRange_literal" ):
                listener.exitRange_literal(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitRange_literal" ):
                return visitor.visitRange_literal(self)
            else:
                return visitor.visitChildren(self)




    def range_literal(self):

        localctx = BESSERActionLanguageParser.Range_literalContext(self, self._ctx, self.state)
        self.enterRule(localctx, 116, self.RULE_range_literal)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 583
            localctx.the_type = self.int_type()
            self.state = 584
            self.match(BESSERActionLanguageParser.T__17)
            self.state = 585
            self.match(BESSERActionLanguageParser.T__12)
            self.state = 586
            localctx.first = self.expression()
            self.state = 587
            self.match(BESSERActionLanguageParser.T__50)
            self.state = 588
            localctx.last = self.expression()
            self.state = 589
            self.match(BESSERActionLanguageParser.T__13)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SymbolContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser
            self.name = None # Token

        def ID(self):
            return self.getToken(BESSERActionLanguageParser.ID, 0)

        def getRuleIndex(self):
            return BESSERActionLanguageParser.RULE_symbol

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSymbol" ):
                listener.enterSymbol(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSymbol" ):
                listener.exitSymbol(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitSymbol" ):
                return visitor.visitSymbol(self)
            else:
                return visitor.visitChildren(self)




    def symbol(self):

        localctx = BESSERActionLanguageParser.SymbolContext(self, self._ctx, self.state)
        self.enterRule(localctx, 118, self.RULE_symbol)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 591
            localctx.name = self.match(BESSERActionLanguageParser.ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx



    def sempred(self, localctx:RuleContext, ruleIndex:int, predIndex:int):
        if self._predicates == None:
            self._predicates = dict()
        self._predicates[29] = self.ternary_sempred
        self._predicates[31] = self.or_sempred
        self._predicates[32] = self.and_sempred
        self._predicates[33] = self.equality_sempred
        self._predicates[34] = self.comparison_sempred
        self._predicates[35] = self.instanceof_sempred
        self._predicates[37] = self.plus_minus_sempred
        self._predicates[38] = self.mult_div_sempred
        self._predicates[39] = self.remain_sempred
        self._predicates[40] = self.primary_sempred
        self._predicates[41] = self.selection_expression_sempred
        pred = self._predicates.get(ruleIndex, None)
        if pred is None:
            raise Exception("No predicate with index:" + str(ruleIndex))
        else:
            return pred(localctx, predIndex)

    def ternary_sempred(self, localctx:TernaryContext, predIndex:int):
            if predIndex == 0:
                return self.precpred(self._ctx, 2)
         

    def or_sempred(self, localctx:OrContext, predIndex:int):
            if predIndex == 1:
                return self.precpred(self._ctx, 2)
         

    def and_sempred(self, localctx:AndContext, predIndex:int):
            if predIndex == 2:
                return self.precpred(self._ctx, 2)
         

    def equality_sempred(self, localctx:EqualityContext, predIndex:int):
            if predIndex == 3:
                return self.precpred(self._ctx, 2)
         

    def comparison_sempred(self, localctx:ComparisonContext, predIndex:int):
            if predIndex == 4:
                return self.precpred(self._ctx, 2)
         

    def instanceof_sempred(self, localctx:InstanceofContext, predIndex:int):
            if predIndex == 5:
                return self.precpred(self._ctx, 2)
         

    def plus_minus_sempred(self, localctx:Plus_minusContext, predIndex:int):
            if predIndex == 6:
                return self.precpred(self._ctx, 2)
         

    def mult_div_sempred(self, localctx:Mult_divContext, predIndex:int):
            if predIndex == 7:
                return self.precpred(self._ctx, 2)
         

    def remain_sempred(self, localctx:RemainContext, predIndex:int):
            if predIndex == 8:
                return self.precpred(self._ctx, 2)
         

    def primary_sempred(self, localctx:PrimaryContext, predIndex:int):
            if predIndex == 9:
                return self.precpred(self._ctx, 1)
         

    def selection_expression_sempred(self, localctx:Selection_expressionContext, predIndex:int):
            if predIndex == 10:
                return self.precpred(self._ctx, 3)
         

            if predIndex == 11:
                return self.precpred(self._ctx, 2)
         

            if predIndex == 12:
                return self.precpred(self._ctx, 1)
         




